@@ -89,7 +89,7 @@ def posix_pipe(fin, fout, delim='\n', buf_size=256,
 # ((opt, type), ... )
 RE_OPTS = re.compile(r'\n {8}(\S+)\s{2,}:\s*([^,]+)')
 # better split method assuming no positional args
-RE_SHLEX = re.compile(r'\s*(?<!\S)--?([^\s=]+)(?:\s*|=|$)')
+RE_SHLEX = re.compile(r'\s*(?<!\S)--?([^\s=]+)(\s+|=|$)')
 
 # TODO: add custom support for some of the following?
 UNSUPPORTED_OPTS = ('iterable', 'gui', 'out', 'file')
@@ -99,7 +99,7 @@ CLI_EXTRA_DOC = r"""
         Extra CLI Options
         -----------------
         name  : type, optional
-             TODO: find out why this is needed.
+            TODO: find out why this is needed.
         delim  : chr, optional
             Delimiting character [default: '\n']. Use '\0' for null.
             N.B.: on Windows systems, Python converts '\n' to '\r\n'.
@@ -128,7 +128,12 @@ def main(fp=sys.stderr, argv=None):
     try:
         log = argv.index('--log')
     except ValueError:
-        logLevel = 'INFO'
+        for i in argv:
+            if i.startswith('--log='):
+                logLevel = i[len('--log='):]
+                break
+        else:
+            logLevel = 'INFO'
     else:
         # argv.pop(log)
         # logLevel = argv.pop(log)
@@ -172,7 +177,7 @@ Options:
         sys.exit(0)
 
     argv = RE_SHLEX.split(' '.join(["tqdm"] + argv))
-    opts = dict(zip(argv[1::2], argv[2::2]))
+    opts = dict(zip(argv[1::3], argv[3::3]))
 
     log.debug(opts)
     opts.pop('log', True)
@@ -195,6 +200,8 @@ Options:
         delim = tqdm_args.pop('delim', '\n')
         delim_per_char = tqdm_args.pop('bytes', False)
         manpath = tqdm_args.pop('manpath', None)
+        stdin = getattr(sys.stdin, 'buffer', sys.stdin)
+        stdout = getattr(sys.stdout, 'buffer', sys.stdout)
         if manpath is not None:
             from os import path
             from shutil import copyfile
@@ -210,14 +217,12 @@ Options:
             tqdm_args.setdefault('unit_divisor', 1024)
             log.debug(tqdm_args)
             with tqdm(**tqdm_args) as t:
-                posix_pipe(sys.stdin, sys.stdout,
-                           '', buf_size, t.update)
+                posix_pipe(stdin, stdout, '', buf_size, t.update)
         elif delim == '\n':
             log.debug(tqdm_args)
-            for i in tqdm(sys.stdin, **tqdm_args):
-                sys.stdout.write(i)
+            for i in tqdm(stdin, **tqdm_args):
+                stdout.write(i)
         else:
             log.debug(tqdm_args)
             with tqdm(**tqdm_args) as t:
-                posix_pipe(sys.stdin, sys.stdout,
-                           delim, buf_size, t.update)
+                posix_pipe(stdin, stdout, delim, buf_size, t.update)
