@@ -151,7 +151,7 @@ class tqdm(Comparable):
             Number ( >= 1) to format.
         suffix  : str, optional
             Post-postfix [default: ''].
-        divisor  : float, optionl
+        divisor  : float, optional
             Divisor between prefixes [default: 1000].
 
         Returns
@@ -503,6 +503,7 @@ class tqdm(Comparable):
                 for inst in cls._instances:
                     # negative `pos` means fixed
                     if hasattr(inst, "pos") and inst.pos > abs(instance.pos):
+                        inst.clear(nolock=True)
                         inst.pos -= 1
                         # TODO: check this doesn't overwrite another fixed bar
             # Kill monitor if no instances are left
@@ -571,8 +572,8 @@ class tqdm(Comparable):
             pandas.core.
             ( frame.DataFrame
             | series.Series
-            | groupby.DataFrameGroupBy
-            | groupby.SeriesGroupBy
+            | groupby.(generic.)DataFrameGroupBy
+            | groupby.(generic.)SeriesGroupBy
             ).progress_apply
 
         A new instance will be create every time `progress_apply` is called,
@@ -601,18 +602,32 @@ class tqdm(Comparable):
         from pandas.core.frame import DataFrame
         from pandas.core.series import Series
         from pandas import Panel
-        try:
-            # pandas>=0.18.0
+        try:  # pandas>=0.18.0
             from pandas.core.window import _Rolling_and_Expanding
         except ImportError:  # pragma: no cover
             _Rolling_and_Expanding = None
-        try:
-            # pandas>=0.23.0
-            from pandas.core.groupby.groupby import DataFrameGroupBy, \
-                SeriesGroupBy, GroupBy, PanelGroupBy
+        try:  # pandas>=0.25.0
+            from pandas.core.groupby.generic import DataFrameGroupBy, \
+                SeriesGroupBy  # , NDFrameGroupBy
         except ImportError:
-            from pandas.core.groupby import DataFrameGroupBy, \
-                SeriesGroupBy, GroupBy, PanelGroupBy
+            try:  # pandas>=0.23.0
+                from pandas.core.groupby.groupby import DataFrameGroupBy, \
+                    SeriesGroupBy
+            except ImportError:
+                from pandas.core.groupby import DataFrameGroupBy, \
+                    SeriesGroupBy
+        try:  # pandas>=0.23.0
+            from pandas.core.groupby.groupby import GroupBy
+        except ImportError:
+            from pandas.core.groupby import GroupBy
+
+        try:  # pandas>=0.23.0
+            from pandas.core.groupby.groupby import PanelGroupBy
+        except ImportError:
+            try:
+                from pandas.core.groupby import PanelGroupBy
+            except ImportError:  # pandas>=0.25.0
+                PanelGroupBy = None
 
         deprecated_t = [tkwargs.pop('deprecated_t', None)]
 
@@ -694,7 +709,8 @@ class tqdm(Comparable):
         DataFrame.progress_applymap = inner_generator('applymap')
 
         Panel.progress_apply = inner_generator()
-        PanelGroupBy.progress_apply = inner_generator()
+        if PanelGroupBy is not None:
+            PanelGroupBy.progress_apply = inner_generator()
 
         GroupBy.progress_apply = inner_generator()
         GroupBy.progress_aggregate = inner_generator('aggregate')
@@ -728,6 +744,7 @@ class tqdm(Comparable):
         leave  : bool, optional
             If [default: True], keeps all traces of the progressbar
             upon termination of iteration.
+            If `None`, will leave only if `position` is `0`.
         file  : `io.TextIOWrapper` or `io.StringIO`, optional
             Specifies where to output the progress messages
             (default: sys.stderr). Uses `file.write(str)` and `file.flush()`
@@ -1173,16 +1190,14 @@ class tqdm(Comparable):
                 return
             raise  # pragma: no cover
 
+        leave = pos == 0 if self.leave is None else self.leave
+
         with self._lock:
-            if self.leave:
-                if self.last_print_n < self.n:
-                    # stats for overall rate (no weighted average)
-                    self.avg_time = None
-                    self.display(pos=pos)
-                if not max([abs(getattr(i, "pos", 0))
-                            for i in self._instances] + [pos]):
-                    # only if not nested (#477)
-                    fp_write('\n')
+            if leave:
+                # stats for overall rate (no weighted average)
+                self.avg_time = None
+                self.display(pos=0)
+                fp_write('\n')
             else:
                 self.display(msg='', pos=pos)
                 if not pos:
