@@ -14,6 +14,18 @@ def _sh(*cmd, **kwargs):
                             **kwargs).communicate()[0].decode('utf-8')
 
 
+class Null(object):
+    def __call__(self, *_, **__):
+        return self
+
+    def __getattr__(self, _):
+        return self
+
+
+IN_DATA_LIST = map(str, _range(int(123)))
+NULL = Null()
+
+
 # WARNING: this should be the last test as it messes with sys.stdin, argv
 @with_setup(pretest, posttest)
 def test_main():
@@ -33,35 +45,64 @@ def test_main():
     _SYS = sys.stdin, sys.argv
 
     with closing(StringIO()) as sys.stdin:
-        sys.argv = ['', '--desc', 'Test CLI-delims',
+        sys.argv = ['', '--desc', 'Test CLI --delim',
                     '--ascii', 'True', '--delim', r'\0', '--buf_size', '64']
-        sys.stdin.write('\0'.join(map(str, _range(int(1e3)))))
+        sys.stdin.write('\0'.join(map(str, _range(int(123)))))
+        #sys.stdin.write(b'\xff')  # TODO
         sys.stdin.seek(0)
         main()
-
-    IN_DATA_LIST = map(str, _range(int(1e3)))
     sys.stdin = IN_DATA_LIST
+
     sys.argv = ['', '--desc', 'Test CLI pipes',
                 '--ascii', 'True', '--unit_scale', 'True']
     import tqdm.__main__  # NOQA
 
-    IN_DATA = '\0'.join(IN_DATA_LIST)
     with closing(StringIO()) as sys.stdin:
+        IN_DATA = '\0'.join(IN_DATA_LIST)
         sys.stdin.write(IN_DATA)
         sys.stdin.seek(0)
-        sys.argv = ['', '--ascii', '--bytes', '--unit_scale', 'False']
+        sys.argv = ['', '--ascii', '--bytes=True', '--unit_scale', 'False']
         with closing(UnicodeIO()) as fp:
             main(fp=fp)
             assert str(len(IN_DATA)) in fp.getvalue()
-
     sys.stdin = IN_DATA_LIST
-    sys.argv = ['', '-ascii', '--unit_scale', 'False',
-                '--desc', 'Test CLI errors']
-    main()
+
+    # test --log
+    with closing(StringIO()) as sys.stdin:
+        sys.stdin.write('\0'.join(map(str, _range(int(123)))))
+        sys.stdin.seek(0)
+        # with closing(UnicodeIO()) as fp:
+        main(argv=['--log', 'DEBUG'], fp=NULL)
+        # assert "DEBUG:" in sys.stdout.getvalue()
+    sys.stdin = IN_DATA_LIST
+
+    # clean up
+    sys.stdin, sys.argv = _SYS
+
+
+def test_manpath():
+    """Test CLI --manpath"""
+    tmp = mkdtemp()
+    man = path.join(tmp, "tqdm.1")
+    assert not path.exists(man)
+    try:
+        main(argv=['--manpath', tmp], fp=NULL)
+    except SystemExit:
+        pass
+    else:
+        raise SystemExit("Expected system exit")
+    assert path.exists(man)
+    rmtree(tmp, True)
+
+
+def test_exceptions():
+    """Test CLI Exceptions"""
+    _SYS = sys.stdin, sys.argv
+    sys.stdin = IN_DATA_LIST
 
     sys.argv = ['', '-ascii', '-unit_scale', '--bad_arg_u_ment', 'foo']
     try:
-        main()
+        main(fp=NULL)
     except TqdmKeyError as e:
         if 'bad_arg_u_ment' not in str(e):
             raise
@@ -70,7 +111,7 @@ def test_main():
 
     sys.argv = ['', '-ascii', '-unit_scale', 'invalid_bool_value']
     try:
-        main()
+        main(fp=NULL)
     except TqdmTypeError as e:
         if 'invalid_bool_value' not in str(e):
             raise
@@ -79,7 +120,7 @@ def test_main():
 
     sys.argv = ['', '-ascii', '--total', 'invalid_int_value']
     try:
-        main()
+        main(fp=NULL)
     except TqdmTypeError as e:
         if 'invalid_int_value' not in str(e):
             raise
@@ -90,30 +131,11 @@ def test_main():
     for i in ('-h', '--help', '-v', '--version'):
         sys.argv = ['', i]
         try:
-            main()
+            main(fp=NULL)
         except SystemExit:
             pass
-
-    # test --manpath
-    tmp = mkdtemp()
-    man = path.join(tmp, "tqdm.1")
-    assert not path.exists(man)
-    try:
-        main(argv=['--manpath', tmp])
-    except SystemExit:
-        pass
-    else:
-        raise SystemExit("Expected system exit")
-    assert path.exists(man)
-    rmtree(tmp, True)
-
-    # test --log
-    with closing(StringIO()) as sys.stdin:
-        sys.stdin.write('\0'.join(map(str, _range(int(1e3)))))
-        sys.stdin.seek(0)
-        # with closing(UnicodeIO()) as fp:
-        main(argv=['--log', 'DEBUG'])
-        # assert "DEBUG:" in sys.stdout.getvalue()
+        else:
+            raise ValueError('expected SystemExit')
 
     # clean up
     sys.stdin, sys.argv = _SYS
